@@ -5,6 +5,7 @@ use warnings;
 use Text::Wrap;
 use Cwd;
 use My::Platform;
+use mtr_report;
 
 # 1. options to support:
 #       --xxx[=ARGS]
@@ -77,7 +78,7 @@ my %debuggers = (
     options => '-f -o {log} {exe} {args}',
   },
   rr => {
-    options => '_RR_TRACE_DIR={log} rr record {exe} {args} --loose-skip-innodb-use-native-aio --loose-innodb-flush-method=fsync',
+    options => '_RR_TRACE_DIR={log} rr record {exe} {args}',
     run => 'env',
     pre => sub {
       ::mtr_error('rr requires kernel.perf_event_paranoid <= 1')
@@ -93,7 +94,7 @@ py
 import subprocess,shlex,time
 valg=subprocess.Popen(shlex.split("""valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --num-callers=16 --quiet --suppressions=valgrind.supp --vgdb-error=0 {exe} {args} --loose-wait-for-pos-timeout=1500"""))
 time.sleep(2)
-gdb.execute("target remote | /usr/lib64/valgrind/../../bin/vgdb --pid=" + str(valg.pid))
+gdb.execute("target remote | vgdb --pid=" + str(valg.pid))
 EEE
     pre => sub {
       my $debug_libraries_path= "/usr/lib/debug";
@@ -108,25 +109,29 @@ EEE
 
 my %opts;
 my %opt_vals;
+my $debugger;
+my $boot_debugger;
+my $client_debugger;
+
 my $help = "\n\nOptions for running debuggers\n\n";
 
 for my $k (sort keys %debuggers) {
   my $v = $debuggers{$k};
   $v = $debuggers{$k} = $debuggers{$v} if not ref $v; # resolve aliases
 
-  sub register_opt($$) {
-    my ($name, $msg) = @_;
-    $opts{"$name=s"} = \$opt_vals{$name};
-    $help .= wrap(sprintf("  %-23s", $name), ' 'x25, "$msg under $name\n");
+  sub register_opt($$$) {
+    my ($prefix, $name, $msg) = @_;
+    $opts{"$prefix$name=s"} = \$opt_vals{$prefix.$name};
+    $help .= wrap(sprintf("  %-23s", $prefix.$name), ' 'x25, "$msg under $name\n");
   }
 
   $v->{script} = '' unless $v->{script};
   $v->{options} =~ s/(\{exe\}|$)/ {options} $&/ unless $v->{options} =~ /\{options\}/;
 
-  register_opt "$k" => "Start mariadbd";
-  register_opt "client-$k" => "Start mariadb-test client";
-  register_opt "boot-$k" => "Start bootstrap server";
-  register_opt "manual-$k" => "Before running test(s) let user manually start mariadbd";
+  register_opt "", $k, "Start mysqld";
+  register_opt "client-", $k, "Start mysqltest client";
+  register_opt "boot-", $k, "Start bootstrap server";
+  register_opt "manual-", "$k", "Before running test(s) let user manually start mariadbd";
 }
 
 sub subst($%) {
@@ -142,7 +147,7 @@ sub do_args($$$$$) {
   my $v = $debuggers{$k};
 
   # on windows mtr args are quoted (for system), otherwise not (for exec)
-  sub quote($) { $_[0] =~ /[; ]/ ? "\"$_[0]\"" : $_[0] }
+  sub quote($) { $_[0] =~ /[; >]/ ? "\"$_[0]\"" : $_[0] }
   sub unquote($) { $_[0] =~ s/^"(.*)"$/$1/; $_[0] }
   sub quote_from_mtr($) { IS_WINDOWS() ? $_[0] : quote($_[0]) }
   sub unquote_for_mtr($) { IS_WINDOWS() ? $_[0] : unquote($_[0]) }
@@ -164,7 +169,7 @@ sub do_args($$$$$) {
   if ($v->{script}) {
     ::mtr_tonewfile($vars{script}, subst($v->{script}, %vars)."\n".$script);
   } elsif ($script) {
-    die "$k is not using a script file, nowhere to write the script \n---\n$script\n---\n";
+    mtr_error "$k is not using a script file, nowhere to write the script \n---\n$script\n---";
   }
 
   my $options = subst($v->{options}, %vars);
@@ -189,16 +194,23 @@ sub help() { $help }
 sub fix_options(@) {
   my $re=join '|', keys %opts;
   $re =~ s/=s//g;
+  # FIXME: what is '=;'? What about ':s' to denote optional argument in register_opt()
   map { $_ . (/^--($re)$/ and '=;') } @_;
 }
 
 sub pre_setup() {
   my $used;
   my $interactive;
+  my %options;
+  my %client_options;
+  my %boot_options;
+
+  my $embedded= $::opt_embedded_server ? ' with --embedded' : '';
+
   for my $k (keys %debuggers) {
     for my $opt ($k, "manual-$k", "boot-$k", "client-$k") {
-      if ($opt_vals{$opt})
-      {
+      my $val= $opt_vals{$opt};
+      if ($val) {
         $used = 1;
         $interactive ||= ($debuggers{$k}->{interactive} ||
                           $debuggers{$k}->{term} ||
@@ -207,10 +219,40 @@ sub pre_setup() {
           $debuggers{$k}->{pre}->();
           delete $debuggers{$k}->{pre};
         }
+        if ($opt eq $k) {
+          $options{$opt}= $val;
+          $client_options{$opt}= $val
+            if $embedded;
+        } elsif ($opt eq "manual-$k") {
+          $options{$opt}= $val;
+        } elsif ($opt eq "boot-$k") {
+          $boot_options{$opt}= $val;
+        } elsif ($opt eq "client-$k") {
+          $client_options{$opt}= $val;
+        }
       }
     }
   }
 
+  if ((keys %options) > 1) {
+    mtr_error "Multiple debuggers specified: ",
+        join (" ", map { "--$_" } keys %options);
+  }
+
+  if ((keys %boot_options) > 1) {
+    mtr_error "Multiple boot debuggers specified: ",
+        join (" ", map { "--$_" } keys %boot_options);
+  }
+
+  if ((keys %client_options) > 1) {
+    mtr_error "Multiple client debuggers specified: ",
+        join (" ", map { "--$_" } keys %client_options);
+  }
+
+  $debugger= (keys %options)[0];
+  $boot_debugger= (keys %boot_options)[0];
+  $client_debugger= (keys %client_options)[0];
+
   if ($used) {
     $ENV{ASAN_OPTIONS}= 'abort_on_error=1:'.($ENV{ASAN_OPTIONS} || '');
     ::mtr_error("Can't use --extern when using debugger") if $ENV{USE_RUNNING_SERVER};
@@ -221,54 +263,26 @@ sub pre_setup() {
     $::opt_suite_timeout= 24 * 60;                              # in minutes
     $::opt_shutdown_timeout= ($interactive ? 24 * 60 : 3) * 60; # in seconds
     $::opt_start_timeout= $::opt_shutdown_timeout;              # in seconds
+    $::opt_debug_sync_timeout= 3000;                            # in seconds
   }
 }
 
 sub setup_boot_args($$$) {
   my ($args, $exe, $input) = @_;
-  my $found;
-
-  for my $k (keys %debuggers) {
-    if ($opt_vals{"boot-$k"}) {
-      die "--boot-$k and --$found cannot be used at the same time\n" if $found;
-
-      $found="boot-$k";
-      do_args($args, $exe, $input, 'bootstrap', $found);
-    }
-  }
+  do_args($args, $exe, $input, 'bootstrap', $boot_debugger)
+    if defined $boot_debugger;
 }
 
 sub setup_client_args($$) {
   my ($args, $exe) = @_;
-  my $found;
-  my $embedded = $::opt_embedded_server ? ' with --embedded' : '';
-
-  for my $k (keys %debuggers) {
-    my @opt_names=("client-$k");
-    push @opt_names, $k if $embedded;
-    for my $opt (@opt_names) {
-      if ($opt_vals{$opt}) {
-        die "--$opt and --$found cannot be used at the same time$embedded\n" if $found;
-        $found=$opt;
-        do_args($args, $exe, IS_WINDOWS() ? 'NUL' : '/dev/null', 'client', $found);
-      }
-    }
-  }
+  do_args($args, $exe, IS_WINDOWS() ? 'NUL' : '/dev/null', 'client', $client_debugger)
+    if defined $client_debugger;
 }
 
 sub setup_args($$$) {
   my ($args, $exe, $type) = @_;
-  my $found;
-
-  for my $k (keys %debuggers) {
-    for my $opt ($k, "manual-$k") {
-      if ($opt_vals{$opt}) {
-        die "--$opt and --$found cannot be used at the same time\n" if $found;
-        $found=$opt;
-        do_args($args, $exe, IS_WINDOWS() ? 'NUL' : '/dev/null', $type, $found);
-      }
-    }
-  }
+  do_args($args, $exe, IS_WINDOWS() ? 'NUL' : '/dev/null', $type, $debugger)
+    if defined $debugger;
 }
 
 1;
